@@ -1,36 +1,36 @@
 #' Parse Formulas of \pkg{brms} Models
-#' 
+#'
 #' Parse formulas objects for use in \pkg{brms}.
-#' 
+#'
 #' @aliases parse_bf
-#' 
+#'
 #' @inheritParams brm
-#' @param check_response Logical; Indicates whether the left-hand side 
-#'   of \code{formula} (i.e. response variables and addition arguments) 
+#' @param check_response Logical; Indicates whether the left-hand side
+#'   of \code{formula} (i.e. response variables and addition arguments)
 #'   should be parsed. If \code{FALSE}, \code{formula} may also be one-sided.
-#' @param resp_rhs_all Logical; Indicates whether to also include response 
-#'   variables on the right-hand side of formula \code{.$allvars}, 
+#' @param resp_rhs_all Logical; Indicates whether to also include response
+#'   variables on the right-hand side of formula \code{.$allvars},
 #'   where \code{.} represents the output of \code{brmsterms}.
 #' @param ... Further arguments passed to or from other methods.
-#'  
-#' @return An object of class \code{brmsterms} or \code{mvbrmsterms} 
-#'   (for multivariate models), which is a \code{list} containing all 
-#'   required information initially stored in \code{formula} 
-#'   in an easier to use format, basically a list of formulas 
+#'
+#' @return An object of class \code{brmsterms} or \code{mvbrmsterms}
+#'   (for multivariate models), which is a \code{list} containing all
+#'   required information initially stored in \code{formula}
+#'   in an easier to use format, basically a list of formulas
 #'   (not an abstract syntax tree).
-#' 
+#'
 #' @details This is the main formula parsing function of \pkg{brms}.
 #'   It should usually not be called directly, but is exported to allow
-#'   package developers making use of the formula syntax implemented 
+#'   package developers making use of the formula syntax implemented
 #'   in \pkg{brms}. As long as no other packages depend on this functions,
 #'   it may be changed without deprecation warnings, when new features make
 #'   this necessary.
-#'   
-#' @seealso 
-#'   \code{\link{brm}}, 
+#'
+#' @seealso
+#'   \code{\link{brm}},
 #'   \code{\link{brmsformula}},
 #'   \code{\link{mvbrmsformula}}
-#' 
+#'
 #' @export
 brmsterms <- function(formula, ...) {
   UseMethod("brmsterms")
@@ -52,7 +52,7 @@ brmsterms.default <- function(formula, ...) {
 
 #' @rdname brmsterms
 #' @export
-brmsterms.brmsformula <- function(formula, check_response = TRUE, 
+brmsterms.brmsformula <- function(formula, check_response = TRUE,
                                   resp_rhs_all = TRUE, ...) {
   x <- validate_formula(formula)
   mv <- isTRUE(x$mv)
@@ -63,22 +63,22 @@ brmsterms.brmsformula <- function(formula, check_response = TRUE,
   y <- nlist(formula, family, mv, rescor, mecor)
   y$cov_ranef <- x$cov_ranef
   class(y) <- "brmsterms"
-  
+
   if (check_response) {
     # extract response variables
     y$respform <- validate_resp_formula(formula, empty_ok = FALSE)
     if (mv) {
-      y$resp <- terms_resp(y$respform) 
+      y$resp <- terms_resp(y$respform)
     } else {
       y$resp <- ""
     }
   }
-  
+
   # extract addition arguments
   adforms <- terms_ad(formula, family, check_response)
   advars <- str2formula(ulapply(adforms, all_vars))
   y$adforms[names(adforms)] <- adforms
-  
+
   # centering would lead to incorrect results for grouped threshold vectors
   # as each threshold vector only affects a subset of observations
   if (!is.null(get_ad_expr(y, "thres", "gr"))) {
@@ -89,7 +89,7 @@ brmsterms.brmsformula <- function(formula, check_response = TRUE,
       attr(x$pforms[[dp]], "center") <- FALSE
     }
   }
-  
+
   # combine the main formula with formulas for the 'mu' parameters
   if (is.mixfamily(family)) {
     mu_dpars <- paste0("mu", seq_along(family$mix))
@@ -107,13 +107,13 @@ brmsterms.brmsformula <- function(formula, check_response = TRUE,
     x$pforms[["mu"]] <- combine_formulas(formula, x$pforms[["mu"]], "mu")
     x$pforms <- move2start(x$pforms, "mu")
   }
-  
+
   # predicted distributional parameters
   resp <- ifelse(mv && !is.null(y$resp), y$resp, "")
   dpars <- intersect(names(x$pforms), valid_dpars(family))
   dpar_forms <- x$pforms[dpars]
   nlpars <- setdiff(names(x$pforms), dpars)
-  
+
   y$dpars <- named_list(dpars)
   for (dp in dpars) {
     if (get_nl(dpar_forms[[dp]])) {
@@ -130,13 +130,13 @@ brmsterms.brmsformula <- function(formula, check_response = TRUE,
     }
     check_cs(y$dpars[[dp]])
   }
-  
+
   y$nlpars <- named_list(nlpars)
   if (length(nlpars)) {
     nlpar_forms <- x$pforms[nlpars]
     for (nlp in nlpars) {
       if (is.null(attr(nlpar_forms[[nlp]], "center"))) {
-        # design matrices of non-linear parameters will not be 
+        # design matrices of non-linear parameters will not be
         # centered by default to make prior specification easier
         attr(nlpar_forms[[nlp]], "center") <- FALSE
       }
@@ -153,7 +153,7 @@ brmsterms.brmsformula <- function(formula, check_response = TRUE,
     unused_nlpars <- setdiff(nlpars, used_nlpars)
     if (length(unused_nlpars)) {
       stop2(
-        "The parameter '", unused_nlpars[1], "' is not a ", 
+        "The parameter '", unused_nlpars[1], "' is not a ",
         "valid distributional or non-linear parameter. ",
         "Did you forget to set 'nl = TRUE'?"
       )
@@ -163,7 +163,7 @@ brmsterms.brmsformula <- function(formula, check_response = TRUE,
     sorted_nlpars <- sort_dependencies(used_nlpars)
     y$nlpars <- y$nlpars[sorted_nlpars]
   }
-  
+
   # fixed distributional parameters
   valid_dpars <- valid_dpars(y)
   inv_fixed_dpars <- setdiff(names(x$pfix), valid_dpars)
@@ -194,19 +194,19 @@ brmsterms.brmsformula <- function(formula, check_response = TRUE,
     y$fdpars[[dp]] <- list(value = x$pfix[[dp]], dpar = dp)
   }
   check_fdpars(y$fdpars)
-  
+
   # make a formula containing all required variables
-  unused_vars <- all_vars(attr(x$formula, "unused"))
+  y$unused <- attr(x$formula, "unused")
   lhsvars <- if (resp_rhs_all) all_vars(y$respform)
   y$allvars <- allvars_formula(
-    lhsvars, advars, lapply(y$dpars, get_allvars), 
+    lhsvars, advars, lapply(y$dpars, get_allvars),
     lapply(y$nlpars, get_allvars), y$time$allvars,
-    unused_vars
+    get_unused_arg_vars(y)
   )
   if (check_response) {
     # add y$respform to the left-hand side of y$allvars
     # avoid using update.formula as it is inefficient for longer formulas
-    formula_allvars <- y$respform 
+    formula_allvars <- y$respform
     formula_allvars[[3]] <- y$allvars[[2]]
     y$allvars <- formula_allvars
   }
@@ -252,7 +252,7 @@ terms_lf <- function(formula) {
   for (t in types) {
     tmp <- do_call(paste0("terms_", t), list(formula))
     if (is.data.frame(tmp) || is.formula(tmp)) {
-      y[[t]] <- tmp 
+      y[[t]] <- tmp
     }
   }
   y$allvars <- allvars_formula(
@@ -282,7 +282,7 @@ terms_nlf <- function(formula, nlpars, resp = "") {
   covars <- setdiff(all_vars, nlpars)
   y$covars <- structure(str2formula(covars), int = FALSE)
   if (!"ac" %in% excluded_term_types(formula)) {
-    y$ac <- terms_ac(attr(formula, "autocor")) 
+    y$ac <- terms_ac(attr(formula, "autocor"))
   }
   y$allvars <- allvars_formula(covars, get_allvars(y$ac))
   environment(y$allvars) <- environment(formula)
@@ -318,12 +318,12 @@ terms_ad <- function(formula, family = NULL, check_response = TRUE) {
           if (!is.na(x[[a]]) && a %in% valid_ads) {
             x[[a]] <- str2formula(x[[a]])
           } else {
-            stop2("Argument '", a, "' is not supported for ", 
+            stop2("Argument '", a, "' is not supported for ",
                   "family '", summary(family), "'.")
-          } 
+          }
         } else if (length(matches) > 1L) {
           stop2("Each addition argument may only be defined once.")
-        } 
+        }
       }
       if (length(ad_terms)) {
         stop2("The following addition terms are invalid:\n",
@@ -386,12 +386,12 @@ terms_re <- function(formula) {
     form <- str2formula(re_parts$lhs[i])
     group <- paste0(gcall$type, collapse(gcall$groups))
     out[[i]] <- data.frame(
-      group = group, gtype = gcall$type, gn = i, 
+      group = group, gtype = gcall$type, gn = i,
       id = gcall$id, type = type[i], cor = gcall$cor,
       stringsAsFactors = FALSE
     )
     out[[i]]$gcall <- list(gcall)
-    out[[i]]$form <- list(form) 
+    out[[i]]$form <- list(form)
     # gather all variables used in the group-level term
     # at this point 'cs' terms are no longer recognized as such
     ftype <- str_if(type[i] %in% "cs", "", type[i])
@@ -426,12 +426,12 @@ terms_cs <- function(formula) {
   out
 }
 
-# extract special effects terms 
+# extract special effects terms
 terms_sp <- function(formula) {
   types <- c("mo", "me", "mi")
   out <- find_terms(formula, types, complete = FALSE)
   if (!length(out)) {
-    return(NULL) 
+    return(NULL)
   }
   uni_mo <- trim_wsp(get_matches_expr(regex_sp("mo"), out))
   uni_me <- trim_wsp(get_matches_expr(regex_sp("me"), out))
@@ -455,7 +455,7 @@ terms_sm <- function(formula) {
     return(NULL)
   }
   if (any(grepl("^(te|ti)\\(", out))) {
-    stop2("Tensor product smooths 'te' and 'ti' are not yet ", 
+    stop2("Tensor product smooths 'te' and 'ti' are not yet ",
           "implemented in brms. Consider using 't2' instead.")
   }
   out <- str2formula(out)
@@ -503,7 +503,7 @@ terms_ac <- function(formula) {
 # extract offset terms
 terms_offset <- function(formula) {
   if (!is.terms(formula)) {
-    formula <- terms(as.formula(formula)) 
+    formula <- terms(as.formula(formula))
   }
   pos <- attr(formula, "offset")
   if (is.null(pos)) {
@@ -540,12 +540,12 @@ terms_resp <- function(formula, check_names = TRUE) {
   if (length(expr) <= 1L) {
     out <- deparse_no_string(expr)
   } else {
-    str_fun <- deparse_no_string(expr[[1]]) 
+    str_fun <- deparse_no_string(expr[[1]])
     used_mvbind <- grepl("^(brms:::?)?mvbind$", str_fun)
     if (used_mvbind) {
       out <- ulapply(expr[-1], deparse_no_string)
     } else {
-      out <- deparse_no_string(expr) 
+      out <- deparse_no_string(expr)
     }
   }
   if (check_names) {
@@ -555,22 +555,22 @@ terms_resp <- function(formula, check_names = TRUE) {
 }
 
 #' Checks if argument is a \code{brmsterms} object
-#' 
+#'
 #' @param x An \R object
-#' 
+#'
 #' @seealso \code{\link[brms:brmsterms]{brmsterms}}
-#' 
+#'
 #' @export
 is.brmsterms <- function(x) {
   inherits(x, "brmsterms")
 }
 
 #' Checks if argument is a \code{mvbrmsterms} object
-#' 
+#'
 #' @param x An \R object
-#' 
+#'
 #' @seealso \code{\link[brms:brmsterms]{brmsterms}}
-#' 
+#'
 #' @export
 is.mvbrmsterms <- function(x) {
   inherits(x, "mvbrmsterms")
@@ -596,14 +596,14 @@ as.brmsterms <- function(x) {
   )
   info <- get(paste0(".family_", families[1]))()
   out$family[names(info)] <- info
-  out$sigma_pred <- any(ulapply(x$terms, 
+  out$sigma_pred <- any(ulapply(x$terms,
     function(x) "sigma" %in% names(x$dpar) || is.formula(x$adforms$se)
   ))
   weight_forms <- rmNULL(lapply(x$terms, function(x) x$adforms$weights))
   if (length(weight_forms)) {
     str_wf <- unique(ulapply(weight_forms, formula2str))
     if (length(str_wf) > 1L) {
-      stop2("All responses should use the same", 
+      stop2("All responses should use the same",
             "weights if 'rescor' is estimated.")
     }
     out$adforms$weights <- weight_forms[[1]]
@@ -638,7 +638,7 @@ avoid_dpars <- function(names, bterms) {
 }
 
 vars_prefix <- function() {
-  c("dpar", "resp", "nlpar") 
+  c("dpar", "resp", "nlpar")
 }
 
 # check and tidy parameter prefixes
@@ -656,11 +656,11 @@ check_prefix <- function(x, keep_mu = FALSE) {
       x[[i]] <- ""
     }
     x[[i]] <- ifelse(
-      !keep_mu & names(x)[i] == "dpar" & x[[i]] %in% "mu", 
+      !keep_mu & names(x)[i] == "dpar" & x[[i]] %in% "mu",
       yes = "", no = x[[i]]
     )
     x[[i]] <- ifelse(
-      keep_mu & names(x)[i] == "dpar" & x[[i]] %in% "", 
+      keep_mu & names(x)[i] == "dpar" & x[[i]] %in% "",
       yes = "mu", no = x[[i]]
     )
   }
@@ -684,7 +684,7 @@ combine_prefix <- function(prefix, keep_mu = FALSE, nlp = FALSE) {
 check_fdpars <- function(x) {
   stopifnot(is.null(x) || is.list(x))
   pos_pars <- c(
-    "sigma", "shape", "nu", "phi", "kappa", 
+    "sigma", "shape", "nu", "phi", "kappa",
     "beta", "disc", "bs", "ndt", "theta"
   )
   prob_pars <- c("zi", "hu", "bias", "quantile")
@@ -711,7 +711,7 @@ allvars_formula <- function(...) {
   invalid_vars <- setdiff(all_vars, make.names(all_vars))
   if (length(invalid_vars)) {
     stop2("The following variable names are invalid: ",
-          collapse_comma(invalid_vars)) 
+          collapse_comma(invalid_vars))
   }
   str2formula(c(out, all_vars))
 }
@@ -769,7 +769,7 @@ is.terms <- function(x) {
 # @param lhs character string to define the left-hand side of the output
 # @param update a flag to indicate whether updating should be allowed.
 #   Defaults to FALSE to maintain backwards compatibility
-# @return a formula object 
+# @return a formula object
 combine_formulas <- function(formula1, formula2, lhs = "", update = FALSE) {
   stopifnot(is.formula(formula1))
   stopifnot(is.null(formula2) || is.formula(formula2))
@@ -795,16 +795,16 @@ combine_formulas <- function(formula1, formula2, lhs = "", update = FALSE) {
   out <- eval2(paste0(lhs, " ~ ", rhs))
   attributes(out)[names(att)] <- att
   out
-} 
+}
 
 # does the formula contain any terms?
 # @return TRUE or FALSE
 has_terms <- function(formula) {
   stopifnot(is.formula(formula))
   terms <- try(terms(rhs(formula)), silent = TRUE)
-  is(terms, "try-error") || 
+  is(terms, "try-error") ||
     length(attr(terms, "term.labels")) ||
-    length(attr(terms, "offset")) 
+    length(attr(terms, "offset"))
 }
 
 # has a linear formula any terms except overall effects?
@@ -917,13 +917,13 @@ all_terms <- function(x) {
 }
 
 # generate a regular expression to extract special terms
-# @param type one or more special term types to be extracted 
+# @param type one or more special term types to be extracted
 regex_sp <- function(type = "all") {
   choices <- c("all", "sp", "sm", "gp", "cs", "mmc", "ac", all_sp_types())
   type <- unique(match.arg(type, choices, several.ok = TRUE))
   funs <- c(
-    sm = "(s|(t2)|(te)|(ti))", 
-    gp = "gp", cs = "cse?", mmc = "mmc", 
+    sm = "(s|(t2)|(te)|(ti))",
+    gp = "gp", cs = "cse?", mmc = "mmc",
     ac = "((arma)|(ar)|(ma)|(cosy)|(sar)|(car)|(fcor))"
   )
   funs[all_sp_types()] <- all_sp_types()
@@ -981,7 +981,7 @@ find_terms <- function(x, type, complete = TRUE, ranef = FALSE) {
 # validate a terms object (or one that can be coerced to it)
 # for use primarily in 'get_model_matrix'
 # @param x any R object
-# @return a (possibly amended) terms object or NULL 
+# @return a (possibly amended) terms object or NULL
 #   if 'x' could not be coerced to a terms object
 validate_terms <- function(x) {
   no_int <- no_int(x)
@@ -999,7 +999,7 @@ validate_terms <- function(x) {
   }
   x
 }
- 
+
 # checks if the formula contains an intercept
 has_intercept <- function(formula) {
   if (is.terms(formula)) {
@@ -1011,7 +1011,7 @@ has_intercept <- function(formula) {
       out <- FALSE
     } else {
       out <- as.logical(attr(try_terms, "intercept"))
-    } 
+    }
   }
   out
 }
@@ -1035,12 +1035,12 @@ has_rsv_intercept <- function(formula, has_intercept = NULL) {
   formula <- try(as.formula(formula), silent = TRUE)
   if (is(formula, "try-error")) {
     return(FALSE)
-  } 
+  }
   if (is.null(has_intercept)) {
     try_terms <- try(terms(formula), silent = TRUE)
     if (is(try_terms, "try-error")) {
       return(FALSE)
-    } 
+    }
     has_intercept <- has_intercept(try_terms)
   }
   .has_rsv_intercept(formula, has_intercept)
@@ -1072,7 +1072,7 @@ check_cs <- function(bterms) {
   stopifnot(is.btl(bterms) || is.btnl(bterms))
   if (has_cs(bterms)) {
     if (!is_equal(dpar_class(bterms$dpar), "mu")) {
-      stop2("Category specific effects are only supported ", 
+      stop2("Category specific effects are only supported ",
             "for the main parameter 'mu'.")
     }
     if (!(is.null(bterms$family) || allow_cs(bterms$family))) {
@@ -1108,6 +1108,31 @@ check_accidental_helper_functions <- function(formula) {
   invisible(TRUE)
 }
 
+# extract names of variables added via the 'unused' argument
+get_unused_arg_vars <- function(x, ...) {
+  UseMethod("get_unused_arg_vars")
+}
+
+#' @export
+get_unused_arg_vars.brmsformula <- function(x, ...) {
+  all_vars(attr(x$formula, "unused"))
+}
+
+#' @export
+get_unused_arg_vars.mvbrmsformula <- function(x, ...) {
+  unique(ulapply(x$forms, get_unused_arg_vars, ...))
+}
+
+#' @export
+get_unused_arg_vars.brmsterms <- function(x, ...) {
+  all_vars(x$unused)
+}
+
+#' @export
+get_unused_arg_vars.mvbrmsterms <- function(x, ...) {
+  unique(ulapply(x$terms, get_unused_arg_vars, ...))
+}
+
 # extract elements from objects
 # @param x object from which to extract elements
 # @param name name of the element to be extracted
