@@ -8,14 +8,15 @@
 #   ad: supported addition arguments
 #   include: names of user-defined Stan functions
 #     to be included in the Stan code
-#   normalized: suffixes of Stan lpdfs or lpmfs which only exist as normalized 
+#   normalized: suffixes of Stan lpdfs or lpmfs which only exist as normalized
 #     versions; can also be "" in which case the family is always normalized
 #   specials: character vector specialties of some families
 
 .family_gaussian <- function() {
   list(
-    links = c("identity", "log", "inverse", "softplus", "squareplus"),
-    dpars = c("mu", "sigma"), type = "real", 
+    links = c("identity", "log", "inverse", "softplus", "squareplus",
+              "logit", "probit", "probit_approx", "cloglog", "cauchit", "softit"),
+    dpars = c("mu", "sigma"), type = "real",
     ybounds = c(-Inf, Inf), closed = c(NA, NA),
     ad = c("weights", "subset", "se", "cens", "trunc", "mi", "index"),
     normalized = c("_time_hom", "_time_het", "_lagsar", "_errorsar", "_fcor"),
@@ -25,8 +26,9 @@
 
 .family_student <- function() {
   list(
-    links = c("identity", "log", "inverse", "softplus", "squareplus"),
-    dpars = c("mu", "sigma", "nu"), type = "real", 
+    links = c("identity", "log", "inverse", "softplus", "squareplus",
+              "logit", "probit", "probit_approx", "cloglog", "cauchit", "softit"),
+    dpars = c("mu", "sigma", "nu"), type = "real",
     ybounds = c(-Inf, Inf), closed = c(NA, NA),
     ad = c("weights", "subset", "se", "cens", "trunc", "mi", "index"),
     include = "fun_logm1.stan",
@@ -37,8 +39,9 @@
 
 .family_skew_normal <- function() {
   list(
-    links = c("identity", "log", "inverse", "softplus", "squareplus"),
-    dpars = c("mu", "sigma", "alpha"), type = "real", 
+    links = c("identity", "log", "inverse", "softplus", "squareplus",
+              "logit", "probit", "probit_approx", "cloglog", "cauchit", "softit"),
+    dpars = c("mu", "sigma", "alpha"), type = "real",
     ybounds = c(-Inf, Inf), closed = c(NA, NA),
     ad = c("weights", "subset", "se", "cens", "trunc", "mi", "index")
   )
@@ -47,47 +50,61 @@
 .family_binomial <- function() {
   list(
     links = c(
-      "logit", "probit", "probit_approx", 
-      "cloglog", "cauchit", "identity"
+      "logit", "probit", "probit_approx", "cloglog",
+      "cauchit", "softit", "identity", "log"
     ),
-    dpars = c("mu"), type = "int", 
+    dpars = c("mu"), type = "int",
     ybounds = c(0, Inf), closed = c(TRUE, NA),
     ad = c("weights", "subset", "trials", "cens", "trunc", "index"),
     specials = "sbi_logit"
   )
 }
 
+.family_beta_binomial <- function() {
+  list(
+    links = c(
+      "logit", "probit", "probit_approx",
+      "cloglog", "cauchit", "softit", "identity"
+    ),
+    dpars = c("mu", "phi"), type = "int",
+    ybounds = c(0, Inf), closed = c(TRUE, NA),
+    ad = c("weights", "subset", "trials", "cens", "trunc", "index")
+  )
+}
+
 .family_bernoulli <- function() {
   list(
     links = c(
-      "logit", "probit", "probit_approx", 
-      "cloglog", "cauchit", "identity"
+      "logit", "probit", "probit_approx", "cloglog",
+      "cauchit", "softit", "identity", "log"
     ),
-    dpars = c("mu"), type = "int", 
+    dpars = c("mu"), type = "int",
     ybounds = c(0, 1), closed = c(TRUE, TRUE),
-    ad = c("weights", "subset", "index"), 
+    ad = c("weights", "subset", "index"),
     specials = c("binary", "sbi_logit")
   )
 }
 
 .family_categorical <- function() {
   list(
-    links = "logit", 
-    dpars = NULL,  # is determined based on the data
-    type = "int", ybounds = c(-Inf, Inf), 
+    links = "logit",
+    dpars = NULL,
+    multi_dpars = "mu",  # size determined by the data
+    type = "int", ybounds = c(-Inf, Inf),
     closed = c(NA, NA),
-    ad = c("weights", "subset", "index"), 
+    ad = c("weights", "subset", "index"),
     specials = c("categorical", "joint_link", "sbi_logit")
   )
 }
 
 .family_multinomial <- function() {
   list(
-    links = "logit", 
-    dpars = NULL,  # is determined based on the data
-    type = "int", ybounds = c(-Inf, Inf), 
+    links = "logit",
+    dpars = NULL,
+    multi_dpars = "mu",  # size determined by the data
+    type = "int", ybounds = c(-Inf, Inf),
     closed = c(NA, NA),
-    ad = c("weights", "subset", "trials", "index"), 
+    ad = c("weights", "subset", "trials", "index"),
     specials = c("multinomial", "joint_link"),
     include = "fun_multinomial_logit.stan",
     normalized = ""
@@ -97,8 +114,8 @@
 .family_beta <- function() {
   list(
     links = c(
-      "logit", "probit", "probit_approx",
-      "cloglog", "cauchit", "identity"
+      "logit", "probit", "probit_approx", "cloglog",
+      "cauchit", "softit", "identity", "log"
     ),
     dpars = c("mu", "phi"), type = "real",
     ybounds = c(0, 1), closed = c(FALSE, FALSE),
@@ -108,12 +125,13 @@
 
 .family_dirichlet <- function() {
   list(
-    links = "logit", 
-    dpars = "phi",  # more dpars are determined based on the data
-    type = "real", ybounds = c(0, 1), 
+    links = "logit",
+    dpars = "phi",
+    multi_dpars = "mu",  # size determined by the data
+    type = "real", ybounds = c(0, 1),
     closed = c(FALSE, FALSE),
-    ad = c("weights", "subset", "index"), 
-    specials = c("dirichlet", "joint_link"),
+    ad = c("weights", "subset", "index"),
+    specials = c("simplex", "joint_link"),
     include = "fun_dirichlet_logit.stan",
     normalized = ""
   )
@@ -121,21 +139,36 @@
 
 .family_dirichlet2 <- function() {
   list(
-    links = c("log", "softplus", "squareplus", "identity", "logm1"), 
-    dpars = NULL,  # is determind based on the data
-    type = "real", ybounds = c(0, 1), 
+    links = c("log", "softplus", "squareplus", "identity", "logm1"),
+    dpars = NULL,
+    multi_dpars = "mu",  # size determined by the data
+    type = "real", ybounds = c(0, 1),
     closed = c(FALSE, FALSE),
-    ad = c("weights", "subset", "index"), 
-    specials = c("dirichlet"),
+    ad = c("weights", "subset", "index"),
+    specials = c("simplex"),
     include = "fun_logm1.stan",
     normalized = ""
   )
 }
 
+.family_logistic_normal <- function() {
+  list(
+    links = "identity",
+    dpars = NULL,
+    multi_dpars = c("mu", "sigma"),  # size determined by the data
+    type = "real", ybounds = c(0, 1),
+    closed = c(FALSE, FALSE),
+    ad = c("weights", "subset", "index"),
+    specials = c("simplex", "logistic_normal", "joint_link"),
+    include = "fun_logistic_normal.stan",
+    normalized = ""
+  )
+}
+
 .family_poisson <- function() {
   list(
     links = c("log", "identity", "sqrt", "softplus", "squareplus"),
-    dpars = c("mu"), type = "int", 
+    dpars = c("mu"), type = "int",
     ybounds = c(0, Inf), closed = c(TRUE, NA),
     ad = c("weights", "subset", "cens", "trunc", "rate", "index"),
     specials = "sbi_log"
@@ -145,7 +178,7 @@
 .family_negbinomial <- function() {
   list(
     links = c("log", "identity", "sqrt", "softplus", "squareplus"),
-    dpars = c("mu", "shape"), type = "int", 
+    dpars = c("mu", "shape"), type = "int",
     ybounds = c(0, Inf), closed = c(TRUE, NA),
     ad = c("weights", "subset", "cens", "trunc", "rate", "index"),
     specials = "sbi_log"
@@ -156,17 +189,17 @@
 .family_negbinomial2 <- function() {
   list(
     links = c("log", "identity", "sqrt", "softplus", "squareplus"),
-    dpars = c("mu", "sigma"), type = "int", 
+    dpars = c("mu", "sigma"), type = "int",
     ybounds = c(0, Inf), closed = c(TRUE, NA),
     ad = c("weights", "subset", "cens", "trunc", "rate", "index"),
     specials = "sbi_log"
   )
 }
- 
+
 .family_geometric <- function() {
   list(
     links = c("log", "identity", "sqrt", "softplus", "squareplus"),
-    dpars = c("mu"), type = "int", 
+    dpars = c("mu"), type = "int",
     ybounds = c(0, Inf), closed = c(TRUE, NA),
     ad = c("weights", "subset", "cens", "trunc", "rate", "index"),
     specials = "sbi_log"
@@ -177,9 +210,9 @@
   list(
     links = c(
       "logit", "probit", "probit_approx",
-      "cloglog", "cauchit", "identity"
+      "cloglog", "cauchit", "softit", "identity"
     ),
-    dpars = c("mu", "shape"), type = "int", 
+    dpars = c("mu", "shape"), type = "int",
     ybounds = c(0, Inf), closed = c(TRUE, NA),
     ad = c("weights", "subset", "cens", "trunc", "index"),
     include = "fun_discrete_weibull.stan"
@@ -189,7 +222,7 @@
 .family_com_poisson <- function() {
   list(
     links = c("log", "identity", "sqrt", "softplus", "squareplus"),
-    dpars = c("mu", "shape"), type = "int", 
+    dpars = c("mu", "shape"), type = "int",
     ybounds = c(0, Inf), closed = c(TRUE, NA),
     ad = c("weights", "subset", "cens", "trunc", "index"),
     include = "fun_com_poisson.stan",
@@ -200,30 +233,27 @@
 .family_gamma <- function() {
   list(
     links = c("log", "identity", "inverse", "softplus", "squareplus"),
-    dpars = c("mu", "shape"), type = "real", 
+    dpars = c("mu", "shape"), type = "real",
     ybounds = c(0, Inf), closed = c(FALSE, NA),
-    ad = c("weights", "subset", "cens", "trunc", "mi", "index"),
-    specials = "transeta"  # see stan_eta_ilink()
+    ad = c("weights", "subset", "cens", "trunc", "mi", "index")
   )
 }
 
 .family_weibull <- function() {
   list(
     links = c("log", "identity", "inverse", "softplus", "squareplus"),
-    dpars = c("mu", "shape"), type = "real", 
+    dpars = c("mu", "shape"), type = "real",
     ybounds = c(0, Inf), closed = c(FALSE, NA),
-    ad = c("weights", "subset", "cens", "trunc", "mi", "index"),
-    specials = "transeta"  # see stan_eta_ilink()
+    ad = c("weights", "subset", "cens", "trunc", "mi", "index")
   )
 }
 
 .family_exponential <- function() {
   list(
     links = c("log", "identity", "inverse", "softplus", "squareplus"),
-    dpars = "mu", type = "real", 
+    dpars = "mu", type = "real",
     ybounds = c(0, Inf), closed = c(FALSE, NA),
-    ad = c("weights", "subset", "cens", "trunc", "mi", "index"),
-    specials = "transeta"  # see stan_eta_ilink()
+    ad = c("weights", "subset", "cens", "trunc", "mi", "index")
   )
 }
 
@@ -233,8 +263,7 @@
     dpars = c("mu", "nu"), type = "real",
     ybounds = c(0, Inf), closed = c(FALSE, NA),
     ad = c("weights", "subset", "cens", "trunc", "mi", "index"),
-    include = "fun_logm1.stan",
-    specials = "transeta"  # see stan_eta_ilink()
+    include = "fun_logm1.stan"
   )
 }
 
@@ -251,7 +280,7 @@
 .family_lognormal <- function() {
   list(
     links = c("identity", "inverse"),
-    dpars = c("mu", "sigma"), type = "real", 
+    dpars = c("mu", "sigma"), type = "real",
     ybounds = c(0, Inf), closed = c(FALSE, NA),
     ad = c("weights", "subset", "cens", "trunc", "mi", "index"),
     specials = "logscale"
@@ -261,7 +290,7 @@
 .family_shifted_lognormal <- function() {
   list(
     links = c("identity", "inverse"),
-    dpars = c("mu", "sigma", "ndt"), type = "real", 
+    dpars = c("mu", "sigma", "ndt"), type = "real",
     ybounds = c(0, Inf), closed = c(FALSE, NA),
     ad = c("weights", "subset", "cens", "trunc", "mi", "index"),
     specials = "logscale"
@@ -291,7 +320,8 @@
 .family_gen_extreme_value <- function() {
   list(
     links = c("identity", "log", "inverse", "softplus", "squareplus"),
-    dpars = c("mu", "sigma", "xi"), type = "real",
+    dpars = c("mu", "sigma", "xi"),
+    tmp_dpars = "xi", type = "real",
     ybounds = c(-Inf, Inf), closed = c(NA, NA),
     ad = c("weights", "subset", "cens", "trunc", "mi", "index"),
     include = c("fun_gen_extreme_value.stan", "fun_scale_xi.stan"),
@@ -346,14 +376,14 @@
 .family_cumulative <- function() {
   list(
     links = c(
-      "logit", "probit", "probit_approx", 
-      "cloglog", "cauchit"
+      "logit", "probit", "probit_approx",
+      "cloglog", "cauchit", "softit"
     ),
-    dpars = c("mu", "disc"), type = "int", 
+    dpars = c("mu", "disc"), type = "int",
     ybounds = c(-Inf, Inf), closed = c(NA, NA),
-    ad = c("weights", "subset", "thres", "cat", "index"), 
+    ad = c("weights", "subset", "thres", "cat", "index"),
     specials = c(
-      "ordinal", "ordered_thres", "thres_minus_eta", 
+      "ordinal", "ordered_thres", "thres_minus_eta",
       "joint_link", "ocs", "sbi_logit"
     ),
     normalized = ""
@@ -363,12 +393,12 @@
 .family_sratio <- function() {
   list(
     links = c(
-      "logit", "probit", "probit_approx", 
-      "cloglog", "cauchit"
+      "logit", "probit", "probit_approx",
+      "cloglog", "cauchit"  # , "softit"
     ),
-    dpars = c("mu", "disc"), type = "int", 
+    dpars = c("mu", "disc"), type = "int",
     ybounds = c(-Inf, Inf), closed = c(NA, NA),
-    ad = c("weights", "subset", "thres", "cat", "index"), 
+    ad = c("weights", "subset", "thres", "cat", "index"),
     specials = c("ordinal", "cs", "thres_minus_eta", "joint_link"),
     normalized = ""
   )
@@ -377,10 +407,10 @@
 .family_cratio <- function() {
   list(
     links = c(
-      "logit", "probit", "probit_approx", 
-      "cloglog", "cauchit"
+      "logit", "probit", "probit_approx",
+      "cloglog", "cauchit"  # , "softit"
     ),
-    dpars = c("mu", "disc"), type = "int", 
+    dpars = c("mu", "disc"), type = "int",
     ybounds = c(-Inf, Inf), closed = c(NA, NA),
     ad = c("weights", "subset", "thres", "cat", "index"),
     specials = c("ordinal", "cs", "eta_minus_thres", "joint_link"),
@@ -391,12 +421,12 @@
 .family_acat <- function() {
   list(
     links = c(
-      "logit", "probit", "probit_approx", 
-      "cloglog", "cauchit"
+      "logit", "probit", "probit_approx",
+      "cloglog", "cauchit", "softit"
     ),
-    dpars = c("mu", "disc"), type = "int", 
+    dpars = c("mu", "disc"), type = "int",
     ybounds = c(-Inf, Inf), closed = c(NA, NA),
-    ad = c("weights", "subset", "thres", "cat", "index"), 
+    ad = c("weights", "subset", "thres", "cat", "index"),
     specials = c("ordinal", "cs", "eta_minus_thres", "joint_link"),
     normalized = ""
   )
@@ -477,8 +507,8 @@
 .family_zero_inflated_binomial <- function() {
   list(
     links = c(
-      "logit", "probit", "probit_approx",
-      "cloglog", "cauchit", "identity"
+      "logit", "probit", "probit_approx", "cloglog",
+      "cauchit", "softit", "identity", "log"
     ),
     dpars = c("mu", "zi"), type = "int",
     ybounds = c(0, Inf), closed = c(TRUE, NA),
@@ -489,11 +519,28 @@
   )
 }
 
+.family_zero_inflated_beta_binomial <- function() {
+  list(
+    links = c(
+      "logit", "probit", "probit_approx", "cloglog",
+      "cauchit", "softit", "identity", "log"
+    ),
+    dpars = c("mu", "phi", "zi"),
+    type = "int",
+    ybounds = c(0, Inf),
+    closed = c(TRUE, NA),
+    ad = c("weights", "subset", "trials", "cens", "trunc", "index"),
+    include = "fun_zero_inflated_beta_binomial.stan",
+    specials = c("sbi_zi_logit"),
+    normalized = ""
+  )
+}
+
 .family_zero_inflated_beta <- function() {
   list(
     links = c(
-      "logit", "probit", "probit_approx",
-      "cloglog", "cauchit", "identity"
+      "logit", "probit", "probit_approx", "cloglog",
+      "cauchit", "softit", "identity", "log"
     ),
     dpars = c("mu", "phi", "zi"), type = "real",
     ybounds = c(0, 1), closed = c(TRUE, FALSE),
@@ -507,8 +554,8 @@
 .family_zero_one_inflated_beta <- function() {
   list(
     links = c(
-      "logit", "probit", "probit_approx",
-      "cloglog", "cauchit", "identity"
+      "logit", "probit", "probit_approx", "cloglog",
+      "cauchit", "softit", "identity", "log"
     ),
     dpars = c("mu", "phi", "zoi", "coi"), type = "real",
     ybounds = c(0, 1), closed = c(TRUE, TRUE),
@@ -521,7 +568,7 @@
 
 .family_custom <- function() {
   list(
-    ad = c("weights", "subset", "se", "cens", "trunc", "trials", 
+    ad = c("weights", "subset", "se", "cens", "trunc", "trials",
            "thres", "cat", "dec", "mi", "index", "vreal", "vint"),
     ybounds = c(-Inf, Inf), closed = c(NA, NA)
   )
