@@ -1,17 +1,17 @@
-# unless otherwise specified, functions return a single character 
+# unless otherwise specified, functions return a single character
 # string defining the likelihood of the model in Stan language
 
 stan_log_lik <- function(x, ...) {
   UseMethod("stan_log_lik")
 }
 
-# Stan code for the model likelihood 
+# Stan code for the model likelihood
 # @param bterms object of class brmsterms
 # @param data data passed by the user
 # @param mix optional mixture component ID
 # @param ptheta are mixing proportions predicted?
 #' @export
-stan_log_lik.family <- function(x, bterms, data, threads, 
+stan_log_lik.family <- function(x, bterms, data, threads, normalize,
                                 mix = "", ptheta = FALSE, ...) {
   stopifnot(is.brmsterms(bterms))
   stopifnot(length(mix) == 1L)
@@ -22,7 +22,7 @@ stan_log_lik.family <- function(x, bterms, data, threads,
   log_lik_fun <- paste0("stan_log_lik_", prepare_family(bterms)$fun)
   ll <- do_call(log_lik_fun, log_lik_args)
   # incorporate other parts into the likelihood
-  args <- nlist(ll, bterms, data, resp, threads, mix, ptheta)
+  args <- nlist(ll, bterms, data, resp, threads, normalize, mix, ptheta)
   if (nzchar(mix)) {
     out <- do_call(stan_log_lik_mix, args)
   } else if (is.formula(bterms$adforms$cens)) {
@@ -35,9 +35,9 @@ stan_log_lik.family <- function(x, bterms, data, threads,
   if (grepl(stan_nn_regex(), out) && !nzchar(mix)) {
     # loop over likelihood if it cannot be vectorized
     out <- paste0(
-      "  for (n in 1:N", resp, ") {\n", 
-      stan_nn_def(threads), 
-      "  ", out, 
+      "  for (n in 1:N", resp, ") {\n",
+      stan_nn_def(threads),
+      "  ", out,
       "  }\n"
     )
   }
@@ -56,13 +56,13 @@ stan_log_lik.mixfamily <- function(x, bterms, threads, ...) {
     sbterms$dpars <- sbterms$dpars[dp_ids == i]
     sbterms$fdpars <- sbterms$fdpars[fdp_ids == i]
     ll[i] <- stan_log_lik(
-      x$mix[[i]], sbterms, mix = i, ptheta = ptheta, 
+      x$mix[[i]], sbterms, mix = i, ptheta = ptheta,
       threads = threads, ...
     )
   }
   resp <- usc(combine_prefix(bterms))
   n <- stan_nn(threads)
-  has_weights <- is.formula(bterms$adforms$weights)  
+  has_weights <- is.formula(bterms$adforms$weights)
   weights <- str_if(has_weights, glue("weights{resp}{n} * "))
   out <- glue(
     "  // likelihood of the mixture model\n",
@@ -88,28 +88,28 @@ stan_log_lik.mvbrmsterms <- function(x, ...) {
   if (x$rescor) {
     out <- stan_log_lik(as.brmsterms(x), ...)
   } else {
-    out <- ulapply(x$terms, stan_log_lik, ...) 
+    out <- ulapply(x$terms, stan_log_lik, ...)
   }
   out
 }
 
 # default likelihood in Stan language
-stan_log_lik_general <- function(ll, bterms, data, threads, resp = "", ...) {
+stan_log_lik_general <- function(ll, bterms, data, threads, normalize, resp = "", ...) {
   stopifnot(is.sdist(ll))
   require_n <- grepl(stan_nn_regex(), ll$args)
   n <- str_if(require_n, stan_nn(threads), stan_slice(threads))
-  lpdf <- stan_log_lik_lpdf_name(bterms)
+  lpdf <- stan_log_lik_lpdf_name(bterms, normalize, dist = ll$dist)
   Y <- stan_log_lik_Y_name(bterms)
   tr <- stan_log_lik_trunc(ll, bterms, data, resp = resp, threads = threads)
   glue("{tp()}{ll$dist}_{lpdf}({Y}{resp}{n}{ll$shift} | {ll$args}){tr};\n")
 }
 
 # censored likelihood in Stan language
-stan_log_lik_cens <- function(ll, bterms, data, threads, resp = "", ...) {
+stan_log_lik_cens <- function(ll, bterms, data, threads, normalize, resp = "", ...) {
   stopifnot(is.sdist(ll))
   s <- wsp(nsp = 4)
   cens <- eval_rhs(bterms$adforms$cens)
-  lpdf <- stan_log_lik_lpdf_name(bterms)
+  lpdf <- stan_log_lik_lpdf_name(bterms, normalize, dist = ll$dist)
   has_weights <- is.formula(bterms$adforms$weights)
   Y <- stan_log_lik_Y_name(bterms)
   n <- stan_nn(threads)
@@ -118,9 +118,9 @@ stan_log_lik_cens <- function(ll, bterms, data, threads, resp = "", ...) {
   tp <- tp()
   out <- glue(
     "// special treatment of censored data\n",
-    s, "if (cens{resp}{n} == 0) {{\n", 
+    s, "if (cens{resp}{n} == 0) {{\n",
     s, "{tp}{w}{ll$dist}_{lpdf}({Y}{resp}{n}{ll$shift} | {ll$args}){tr};\n",
-    s, "}} else if (cens{resp}{n} == 1) {{\n",         
+    s, "}} else if (cens{resp}{n} == 1) {{\n",
     s, "{tp}{w}{ll$dist}_lccdf({Y}{resp}{n}{ll$shift} | {ll$args}){tr};\n",
     s, "}} else if (cens{resp}{n} == -1) {{\n",
     s, "{tp}{w}{ll$dist}_lcdf({Y}{resp}{n}{ll$shift} | {ll$args}){tr};\n"
@@ -129,9 +129,9 @@ stan_log_lik_cens <- function(ll, bterms, data, threads, resp = "", ...) {
     # interval censoring is required
     str_add(out) <- glue(
       s, "}} else if (cens{resp}{n} == 2) {{\n",
-      s, "{tp}{w}log_diff_exp(\n", 
-      s, "    {ll$dist}_lcdf(rcens{resp}{n}{ll$shift} | {ll$args}),\n", 
-      s, "    {ll$dist}_lcdf({Y}{resp}{n}{ll$shift} | {ll$args})\n", 
+      s, "{tp}{w}log_diff_exp(\n",
+      s, "    {ll$dist}_lcdf(rcens{resp}{n}{ll$shift} | {ll$args}),\n",
+      s, "    {ll$dist}_lcdf({Y}{resp}{n}{ll$shift} | {ll$args})\n",
       s, "  ){tr};\n"
     )
   }
@@ -140,28 +140,28 @@ stan_log_lik_cens <- function(ll, bterms, data, threads, resp = "", ...) {
 }
 
 # weighted likelihood in Stan language
-stan_log_lik_weights <- function(ll, bterms, data, threads, resp = "", ...) {
+stan_log_lik_weights <- function(ll, bterms, data, threads, normalize, resp = "", ...) {
   stopifnot(is.sdist(ll))
   tr <- stan_log_lik_trunc(ll, bterms, data, resp = resp, threads = threads)
-  lpdf <- stan_log_lik_lpdf_name(bterms)
+  lpdf <- stan_log_lik_lpdf_name(bterms, normalize, dist = ll$dist)
   Y <- stan_log_lik_Y_name(bterms)
   n <- stan_nn(threads)
   glue(
-    "{tp()}weights{resp}{n} * ({ll$dist}_{lpdf}", 
+    "{tp()}weights{resp}{n} * ({ll$dist}_{lpdf}",
     "({Y}{resp}{n}{ll$shift} | {ll$args}){tr});\n"
   )
 }
 
 # likelihood of a single mixture component
-stan_log_lik_mix <- function(ll, bterms, data, mix, ptheta, threads, 
-                             resp = "", ...) {
+stan_log_lik_mix <- function(ll, bterms, data, mix, ptheta, threads,
+                             normalize, resp = "", ...) {
   stopifnot(is.sdist(ll))
   theta <- str_if(ptheta,
-    glue("theta{mix}{resp}[n]"), 
+    glue("theta{mix}{resp}[n]"),
     glue("log(theta{mix}{resp})")
   )
   tr <- stan_log_lik_trunc(ll, bterms, data, resp = resp, threads = threads)
-  lpdf <- stan_log_lik_lpdf_name(bterms)
+  lpdf <- stan_log_lik_lpdf_name(bterms, normalize, dist = ll$dist)
   Y <- stan_log_lik_Y_name(bterms)
   n <- stan_nn(threads)
   if (is.formula(bterms$adforms$cens)) {
@@ -170,10 +170,10 @@ stan_log_lik_mix <- function(ll, bterms, data, mix, ptheta, threads,
     s <- wsp(nsp = 4)
     out <- glue(
       "// special treatment of censored data\n",
-      s, "if (cens{resp}{n} == 0) {{\n", 
-      s, "  ps[{mix}] = {theta} + ", 
+      s, "if (cens{resp}{n} == 0) {{\n",
+      s, "  ps[{mix}] = {theta} + ",
       "{ll$dist}_{lpdf}({Y}{resp}{n}{ll$shift} | {ll$args}){tr};\n",
-      s, "}} else if (cens{resp}{n} == 1) {{\n",         
+      s, "}} else if (cens{resp}{n} == 1) {{\n",
       s, "  ps[{mix}] = {theta} + ",
       "{ll$dist}_lccdf({Y}{resp}{n}{ll$shift} | {ll$args}){tr};\n",
       s, "}} else if (cens{resp}{n} == -1) {{\n",
@@ -184,25 +184,25 @@ stan_log_lik_mix <- function(ll, bterms, data, mix, ptheta, threads,
       # interval censoring is required
       str_add(out) <- glue(
         s, "}} else if (cens{resp}{n} == 2) {{\n",
-        s, "  ps[{mix}] = {theta} + log_diff_exp(\n", 
-        s, "    {ll$dist}_lcdf(rcens{resp}{n}{ll$shift} | {ll$args}),\n", 
-        s, "    {ll$dist}_lcdf({Y}{resp}{n}{ll$shift} | {ll$args})\n", 
+        s, "  ps[{mix}] = {theta} + log_diff_exp(\n",
+        s, "    {ll$dist}_lcdf(rcens{resp}{n}{ll$shift} | {ll$args}),\n",
+        s, "    {ll$dist}_lcdf({Y}{resp}{n}{ll$shift} | {ll$args})\n",
         s, "  ){tr};\n"
       )
     }
     str_add(out) <- glue(s, "}}\n")
   } else {
     out <- glue(
-      "ps[{mix}] = {theta} + ", 
+      "ps[{mix}] = {theta} + ",
       "{ll$dist}_{lpdf}({Y}{resp}{n}{ll$shift} | {ll$args}){tr};\n"
-    ) 
+    )
   }
   out
 }
 
 # truncated part of the likelihood
 # @param short use the T[, ] syntax?
-stan_log_lik_trunc <- function(ll, bterms, data, threads,resp = "", 
+stan_log_lik_trunc <- function(ll, bterms, data, threads, resp = "",
                                short = FALSE) {
   stopifnot(is.sdist(ll))
   bounds <- trunc_bounds(bterms, data = data)
@@ -220,20 +220,33 @@ stan_log_lik_trunc <- function(ll, bterms, data, threads,resp = "",
     # truncation making use of _lcdf functions
     ms <- paste0(" -\n", wsp(nsp = 6))
     if (any(bounds$lb > -Inf) && !any(bounds$ub < Inf)) {
-      out <- glue("{ms}{ll$dist}_lccdf({lb} | {ll$args})")
+      out <- glue("{ms}{ll$dist}_lccdf({lb}{ll$shift} | {ll$args})")
     } else if (!any(bounds$lb > -Inf) && any(bounds$ub < Inf)) {
-      out <- glue("{ms}{ll$dist}_lcdf({ub} | {ll$args})")
+      out <- glue("{ms}{ll$dist}_lcdf({ub}{ll$shift} | {ll$args})")
     } else if (any(bounds$lb > -Inf) && any(bounds$ub < Inf)) {
-      trr <- glue("{ll$dist}_lcdf({ub} | {ll$args})")
-      trl <- glue("{ll$dist}_lcdf({lb} | {ll$args})")
+      trr <- glue("{ll$dist}_lcdf({ub}{ll$shift} | {ll$args})")
+      trl <- glue("{ll$dist}_lcdf({lb}{ll$shift} | {ll$args})")
       out <- glue("{ms}log_diff_exp({trr}, {trl})")
     }
   }
   out
 }
 
-stan_log_lik_lpdf_name <- function(bterms) {
-  ifelse(use_int(bterms$family), "lpmf", "lpdf")
+stan_log_lik_lpdf_name <- function(bterms, normalize, dist = NULL) {
+  if (!is.null(dist) && !normalize) {
+    # some Stan lpdfs or lpmfs only exist as normalized versions
+    always_normalized <- always_normalized(bterms)
+    if (length(always_normalized)) {
+      always_normalized <- paste0(escape_all(always_normalized), "$")
+      normalize <- any(ulapply(always_normalized, grepl, x = dist))
+    }
+  }
+  if (normalize) {
+    out <- ifelse(use_int(bterms$family), "lpmf", "lpdf")
+  } else {
+    out <- ifelse(use_int(bterms$family), "lupmf", "lupdf")
+  }
+  out
 }
 
 stan_log_lik_Y_name <- function(bterms) {
@@ -244,11 +257,16 @@ stan_log_lik_Y_name <- function(bterms) {
 # @param reqn will the likelihood be wrapped in a loop over n?
 # @param dpars optional names of distributional parameters to be prepared
 #   if not specified will prepare all distributional parameters
-stan_log_lik_dpars <- function(bterms, reqn, resp = "", mix = "", dpars = NULL) {
+stan_log_lik_dpars <- function(bterms, reqn, resp = "", mix = "", dpars = NULL,
+                               type = NULL) {
   if (is.null(dpars)) {
-    dpars <- paste0(valid_dpars(bterms), mix)
+    dpars <- paste0(valid_dpars(bterms, type = type), mix)
   }
-  is_pred <- dpars %in% c("mu", names(bterms$dpars))
+  pred_dpars <- names(bterms$dpars)
+  if (is_equal(type, "multi")) {
+    pred_dpars <- unique(dpar_class(pred_dpars, bterms))
+  }
+  is_pred <- dpars %in% pred_dpars
   out <- paste0(dpars, resp, ifelse(reqn & is_pred, "[n]", ""))
   named_list(dpars, out)
 }
@@ -275,14 +293,14 @@ stan_log_lik_dpar_usc_logit <- function(dpar, bterms) {
 }
 
 # add 'se' to 'sigma' within the Stan likelihood
-stan_log_lik_add_se <- function(sigma, bterms, reqn, resp = "", 
+stan_log_lik_add_se <- function(sigma, bterms, reqn, resp = "",
                                 threads = NULL) {
   if (!is.formula(bterms$adforms$se)) {
-    return(sigma) 
+    return(sigma)
   }
   nse <- str_if(reqn, stan_nn(threads), stan_slice(threads))
   if (no_sigma(bterms)) {
-    sigma <- glue("se{resp}{nse}") 
+    sigma <- glue("se{resp}{nse}")
   } else {
     sigma <- glue("sqrt(square({sigma}) + se2{resp}{nse})")
   }
@@ -291,10 +309,14 @@ stan_log_lik_add_se <- function(sigma, bterms, reqn, resp = "",
 
 # multiply 'dpar' by the 'rate' denominator within the Stan likelihood
 # @param log add the rate denominator on the log scale if sensible?
-stan_log_lik_multiply_rate_denom <- function(dpar, bterms, reqn, resp = "", 
-                                             log = FALSE) {
+stan_log_lik_multiply_rate_denom <- function(dpar, bterms, reqn, resp = "",
+                                             log = FALSE, transform = NULL) {
+  dpar_transform <- dpar
+  if (!is.null(transform)) {
+    dpar_transform <- glue("{transform}({dpar})")
+  }
   if (!is.formula(bterms$adforms$rate)) {
-    return(dpar)
+    return(dpar_transform)
   }
   ndenom <- str_if(reqn, "[n]")
   denom <- glue("denom{resp}{ndenom}")
@@ -306,7 +328,7 @@ stan_log_lik_multiply_rate_denom <- function(dpar, bterms, reqn, resp = "",
     is_pred <- dpar %in% c("mu", names(bterms$dpars))
     operator <- str_if(reqn || !is_pred, "*", ".*")
   }
-  glue("{dpar} {operator} {denom}")
+  glue("{dpar_transform} {operator} {denom}")
 }
 
 # check if the log-liklihood needs to be adjused
@@ -330,7 +352,7 @@ stan_log_lik_gaussian <- function(bterms, resp = "", mix = "", threads = NULL,
     reqn <- stan_log_lik_adj(bterms) || nzchar(mix)
     p <- stan_log_lik_dpars(bterms, reqn, resp, mix)
     p$sigma <- stan_log_lik_add_se(p$sigma, bterms, reqn, resp, threads)
-    out <- sdist("normal", p$mu, p$sigma) 
+    out <- sdist("normal", p$mu, p$sigma)
   }
   out
 }
@@ -350,7 +372,7 @@ stan_log_lik_gaussian_time <- function(bterms, resp = "", mix = "", ...) {
   v <- c("chol_cor", "se2", "nobs_tg", "begin_tg", "end_tg")
   p[v] <- as.list(paste0(v, resp))
   sfx <- str_if("sigma" %in% names(bterms$dpars), "het", "hom")
-  sdist(glue("normal_time_{sfx}"), 
+  sdist(glue("normal_time_{sfx}"),
     p$mu, p$sigma, p$chol_cor, p$se2,
     p$nobs_tg, p$begin_tg, p$end_tg
   )
@@ -376,7 +398,7 @@ stan_log_lik_gaussian_lagsar <- function(bterms, resp = "", mix = "",
   sdist("normal_lagsar", p$mu, p$sigma, p$lagsar, p$Msar, p$eigenMsar)
 }
 
-stan_log_lik_gaussian_errorsar <- function(bterms, resp = "", mix = "", 
+stan_log_lik_gaussian_errorsar <- function(bterms, resp = "", mix = "",
                                            threads = NULL, ...) {
   p <- stan_log_lik_dpars(bterms, FALSE, resp, mix)
   p$sigma <- stan_log_lik_add_se(p$sigma, bterms, FALSE, resp, threads)
@@ -409,7 +431,7 @@ stan_log_lik_student_time <- function(bterms, resp = "", mix = "", ...) {
   v <- c("chol_cor", "se2", "nobs_tg", "begin_tg", "end_tg")
   p[v] <- as.list(paste0(v, resp))
   sfx <- str_if("sigma" %in% names(bterms$dpars), "het", "hom")
-  sdist(glue("student_t_time_{sfx}"), 
+  sdist(glue("student_t_time_{sfx}"),
     p$nu, p$mu, p$sigma, p$chol_cor, p$se2,
     p$nobs_tg, p$begin_tg, p$end_tg
   )
@@ -432,7 +454,7 @@ stan_log_lik_student_lagsar <- function(bterms, resp = "", mix = "",
   p$sigma <- stan_log_lik_add_se(p$sigma, bterms, FALSE, resp, threads)
   v <- c("lagsar", "Msar", "eigenMsar")
   p[v] <- as.list(paste0(v, resp))
-  sdist("student_t_lagsar", p$nu, p$mu, p$sigma, 
+  sdist("student_t_lagsar", p$nu, p$mu, p$sigma,
         p$lagsar, p$Msar, p$eigenMsar)
 }
 
@@ -442,7 +464,7 @@ stan_log_lik_student_errorsar <- function(bterms, resp = "", mix = "",
   p$sigma <- stan_log_lik_add_se(p$sigma, bterms, FALSE, resp, threads)
   v <- c("errorsar", "Msar", "eigenMsar")
   p[v] <- as.list(paste0(v, resp))
-  sdist("student_t_errorsar", p$nu, p$mu, p$sigma, 
+  sdist("student_t_errorsar", p$nu, p$mu, p$sigma,
         p$errorsar, p$Msar, p$eigenMsar)
 }
 
@@ -475,7 +497,7 @@ stan_log_lik_skew_normal <- function(bterms, resp = "", mix = "",
   sdist("skew_normal", p$mu, p$omega, p$alpha)
 }
 
-stan_log_lik_poisson <- function(bterms, resp = "", mix = "", threads = 1,
+stan_log_lik_poisson <- function(bterms, resp = "", mix = "", threads = NULL,
                                  ...) {
   if (use_glm_primitive(bterms)) {
     p <- args_glm_primitive(bterms$dpars$mu, resp = resp, threads = threads)
@@ -490,7 +512,7 @@ stan_log_lik_poisson <- function(bterms, resp = "", mix = "", threads = 1,
   out
 }
 
-stan_log_lik_negbinomial <- function(bterms, resp = "", mix = "", threads = 1,
+stan_log_lik_negbinomial <- function(bterms, resp = "", mix = "", threads = NULL,
                                      ...) {
   if (use_glm_primitive(bterms)) {
     p <- args_glm_primitive(bterms$dpars$mu, resp = resp, threads = threads)
@@ -507,7 +529,27 @@ stan_log_lik_negbinomial <- function(bterms, resp = "", mix = "", threads = 1,
   out
 }
 
-stan_log_lik_geometric <- function(bterms, resp = "", mix = "", threads = 1, 
+stan_log_lik_negbinomial2 <- function(bterms, resp = "", mix = "", threads = NULL,
+                                      ...) {
+  if (use_glm_primitive(bterms)) {
+    p <- args_glm_primitive(bterms$dpars$mu, resp = resp, threads = threads)
+    p$sigma <- paste0("sigma", resp)
+    p$shape <- paste0("inv(", p$sigma, ")")
+    out <- sdist("neg_binomial_2_log_glm", p$x, p$alpha, p$beta, p$shape)
+  } else {
+    reqn <- stan_log_lik_adj(bterms) || nzchar(mix)
+    p <- stan_log_lik_dpars(bterms, reqn, resp, mix)
+    p$mu <- stan_log_lik_multiply_rate_denom(p$mu, bterms, reqn, resp, log = TRUE)
+    p$shape <- stan_log_lik_multiply_rate_denom(
+      p$sigma, bterms, reqn, resp, transform = "inv"
+    )
+    lpdf <- stan_log_lik_simple_lpdf("neg_binomial_2", "log", bterms)
+    out <- sdist(lpdf, p$mu, p$shape)
+  }
+  out
+}
+
+stan_log_lik_geometric <- function(bterms, resp = "", mix = "", threads = NULL,
                                    ...) {
   if (use_glm_primitive(bterms)) {
     p <- args_glm_primitive(bterms$dpars$mu, resp = resp, threads = threads)
@@ -524,7 +566,7 @@ stan_log_lik_geometric <- function(bterms, resp = "", mix = "", threads = 1,
   }
 }
 
-stan_log_lik_binomial <- function(bterms, resp = "", mix = "", threads = 1, 
+stan_log_lik_binomial <- function(bterms, resp = "", mix = "", threads = NULL,
                                   ...) {
   reqn <- stan_log_lik_adj(bterms) || nzchar(mix)
   p <- stan_log_lik_dpars(bterms, reqn, resp, mix)
@@ -534,7 +576,19 @@ stan_log_lik_binomial <- function(bterms, resp = "", mix = "", threads = 1,
   sdist(lpdf, p$trials, p$mu)
 }
 
-stan_log_lik_bernoulli <- function(bterms, resp = "", mix = "", threads = 1, 
+stan_log_lik_beta_binomial <- function(bterms, resp = "", mix = "",
+                                       threads = NULL, ...) {
+  p <- stan_log_lik_dpars(bterms, TRUE, resp, mix)
+  n <- stan_nn(threads)
+  sdist(
+    "beta_binomial",
+    paste0("trials", resp, n),
+    paste0(p$mu, " * ", p$phi),
+    paste0("(1 - ", p$mu, ") * ", p$phi)
+  )
+}
+
+stan_log_lik_bernoulli <- function(bterms, resp = "", mix = "", threads = NULL,
                                    ...) {
   if (use_glm_primitive(bterms)) {
     p <- args_glm_primitive(bterms$dpars$mu, resp = resp, threads = threads)
@@ -560,27 +614,39 @@ stan_log_lik_com_poisson <- function(bterms, resp = "", mix = "", ...) {
 }
 
 stan_log_lik_gamma <- function(bterms, resp = "", mix = "", ...) {
-  reqn <- stan_log_lik_adj(bterms) || nzchar(mix)
+  reqn <- stan_log_lik_adj(bterms) || nzchar(mix) ||
+    paste0("shape", mix) %in% names(bterms$dpars)
   p <- stan_log_lik_dpars(bterms, reqn, resp, mix)
-  sdist("gamma", p$shape, p$mu)
+  # Stan uses shape-rate parameterization with rate = shape / mean
+  div_op <- str_if(reqn, " / ", " ./ ")
+  sdist("gamma", p$shape, paste0(p$shape, div_op, p$mu))
 }
 
 stan_log_lik_exponential <- function(bterms, resp = "", mix = "", ...) {
   reqn <- stan_log_lik_adj(bterms) || nzchar(mix)
   p <- stan_log_lik_dpars(bterms, reqn, resp, mix)
-  sdist("exponential", p$mu)
+  # Stan uses rate parameterization with rate = 1 / mean
+  sdist("exponential", paste0("inv(", p$mu, ")"))
 }
 
 stan_log_lik_weibull <- function(bterms, resp = "", mix = "", ...) {
   reqn <- stan_log_lik_adj(bterms) || nzchar(mix)
   p <- stan_log_lik_dpars(bterms, reqn, resp, mix)
-  sdist("weibull", p$shape, p$mu)
+  # Stan uses shape-scale parameterization for weibull
+  need_dot_div <- !reqn && paste0("shape", mix) %in% names(bterms$dpars)
+  div_op <- str_if(need_dot_div, " ./ ", " / ")
+  p$scale <- paste0(p$mu, div_op, "tgamma(1 + 1", div_op, p$shape, ")")
+  sdist("weibull", p$shape, p$scale)
 }
 
 stan_log_lik_frechet <- function(bterms, resp = "", mix = "", ...) {
   reqn <- stan_log_lik_adj(bterms) || nzchar(mix)
   p <- stan_log_lik_dpars(bterms, reqn, resp, mix)
-  sdist("frechet", p$nu, p$mu)
+  # Stan uses shape-scale parameterization for frechet
+  need_dot_div <- !reqn && paste0("nu", mix) %in% names(bterms$dpars)
+  div_op <- str_if(need_dot_div, " ./ ", " / ")
+  p$scale <- paste0(p$mu, div_op, "tgamma(1 - 1", div_op, p$nu, ")")
+  sdist("frechet", p$nu, p$scale)
 }
 
 stan_log_lik_gen_extreme_value <- function(bterms, resp = "", mix = "", ...) {
@@ -592,7 +658,7 @@ stan_log_lik_exgaussian <- function(bterms, resp = "", mix = "", ...) {
   reqn <- stan_log_lik_adj(bterms) || nzchar(mix)
   p <- stan_log_lik_dpars(bterms, reqn, resp, mix)
   sdist(
-    "exp_mod_normal", paste0(p$mu, " - ", p$beta), 
+    "exp_mod_normal", paste0(p$mu, " - ", p$beta),
     p$sigma, paste0("inv(", p$beta, ")")
   )
 }
@@ -601,14 +667,16 @@ stan_log_lik_inverse.gaussian <- function(bterms, resp = "", mix = "", ...) {
   reqn <- stan_log_lik_adj(bterms) || nzchar(mix) ||
     glue("shape{mix}") %in% names(bterms$dpars)
   p <- stan_log_lik_dpars(bterms, reqn, resp, mix)
-  lpdf <- paste0("inv_gaussian", if (!reqn) "vector")
+  lpdf <- paste0("inv_gaussian", if (!reqn) "_vector")
   n <- str_if(reqn, "[n]")
   sdist(lpdf, p$mu, p$shape)
 }
 
-stan_log_lik_wiener <- function(bterms, resp = "", mix = "", ...) {
+stan_log_lik_wiener <- function(bterms, resp = "", mix = "", threads = NULL,
+                                ...) {
   p <- stan_log_lik_dpars(bterms, TRUE, resp, mix)
-  p$dec <- paste0("dec", resp, "[n]")
+  n <- stan_nn(threads)
+  p$dec <- paste0("dec", resp, n)
   sdist("wiener_diffusion", p$dec, p$bs, p$ndt, p$bias, p$mu)
 }
 
@@ -617,7 +685,7 @@ stan_log_lik_beta <- function(bterms, resp = "", mix = "", ...) {
     paste0("phi", mix) %in% names(bterms$dpars)
   p <- stan_log_lik_dpars(bterms, reqn, resp, mix)
   sdist("beta",
-    paste0(p$mu, " * ", p$phi), 
+    paste0(p$mu, " * ", p$phi),
     paste0("(1 - ", p$mu, ") * ", p$phi)
   )
 }
@@ -630,7 +698,7 @@ stan_log_lik_von_mises <- function(bterms, resp = "", mix = "", ...) {
   sdist(lpdf, p$mu, p$kappa)
 }
 
-stan_log_lik_cox <- function(bterms, resp = "", mix = "", threads = 1,
+stan_log_lik_cox <- function(bterms, resp = "", mix = "", threads = NULL,
                              ...) {
   p <- stan_log_lik_dpars(bterms, TRUE, resp, mix)
   n <- stan_nn(threads)
@@ -644,51 +712,70 @@ stan_log_lik_cox <- function(bterms, resp = "", mix = "", threads = 1,
 }
 
 stan_log_lik_cumulative <- function(bterms, resp = "", mix = "",
-                                    threads = 1, ...) {
+                                    threads = NULL, ...) {
   if (use_glm_primitive(bterms, allow_special_terms = FALSE)) {
     p <- args_glm_primitive(bterms$dpars$mu, resp = resp, threads = threads)
     out <- sdist("ordered_logistic_glm", p$x, p$beta, p$alpha)
     return(out)
-  } 
-  stan_log_lik_ordinal(bterms, resp, mix)
+  }
+  stan_log_lik_ordinal(bterms, resp, mix, threads, ...)
 }
 
-stan_log_lik_sratio <- function(bterms, resp = "", mix = "", ...) {
-  stan_log_lik_ordinal(bterms, resp, mix)
+stan_log_lik_sratio <- function(bterms, resp = "", mix = "",
+                                threads = NULL, ...) {
+  stan_log_lik_ordinal(bterms, resp, mix, threads, ...)
 }
 
-stan_log_lik_cratio <- function(bterms, resp = "", mix = "", ...) {
-  stan_log_lik_ordinal(bterms, resp, mix)
+stan_log_lik_cratio <- function(bterms, resp = "", mix = "",
+                                threads = NULL, ...) {
+  stan_log_lik_ordinal(bterms, resp, mix, threads, ...)
 }
 
-stan_log_lik_acat <- function(bterms, resp = "", mix = "", ...) {
-  stan_log_lik_ordinal(bterms, resp, mix)
+stan_log_lik_acat <- function(bterms, resp = "", mix = "",
+                              threads = NULL, ...) {
+  stan_log_lik_ordinal(bterms, resp, mix, threads, ...)
 }
 
 stan_log_lik_categorical <- function(bterms, resp = "", mix = "", ...) {
   stopifnot(bterms$family$link == "logit")
   stopifnot(!isTRUE(nzchar(mix)))  # mixture models are not allowed
-  p <- stan_log_lik_dpars(bterms, TRUE, resp, mix, dpars = "mu")
+  p <- stan_log_lik_dpars(bterms, TRUE, resp, mix, dpars = "mu", type = "multi")
   sdist("categorical_logit", p$mu)
 }
 
 stan_log_lik_multinomial <- function(bterms, resp = "", mix = "", ...) {
   stopifnot(bterms$family$link == "logit")
   stopifnot(!isTRUE(nzchar(mix)))  # mixture models are not allowed
-  p <- stan_log_lik_dpars(bterms, TRUE, resp, mix, dpars = "mu")
-  sdist("multinomial_logit", p$mu)
+  p <- stan_log_lik_dpars(bterms, TRUE, resp, mix, dpars = "mu", type = "multi")
+  sdist("multinomial_logit2", p$mu)
 }
 
 stan_log_lik_dirichlet <- function(bterms, resp = "", mix = "", ...) {
   stopifnot(bterms$family$link == "logit")
   stopifnot(!isTRUE(nzchar(mix)))  # mixture models are not allowed
-  mu <- stan_log_lik_dpars(bterms, TRUE, resp, mix, dpars = "mu")$mu
+  mu <- stan_log_lik_dpars(bterms, TRUE, resp, mix, dpars = "mu", type = "multi")$mu
   reqn <- glue("phi{mix}") %in% names(bterms$dpars)
   phi <- stan_log_lik_dpars(bterms, reqn, resp, mix, dpars = "phi")$phi
   sdist("dirichlet_logit", mu, phi)
 }
 
-stan_log_lik_ordinal <- function(bterms, resp = "", mix = "", ...) {
+stan_log_lik_dirichlet2 <- function(bterms, resp = "", mix = "", ...) {
+  stopifnot(!isTRUE(nzchar(mix)))  # mixture models are not allowed
+  mu <- stan_log_lik_dpars(bterms, TRUE, resp, mix, dpars = "mu", type = "multi")$mu
+  sdist("dirichlet", mu)
+}
+
+stan_log_lik_logistic_normal <- function(bterms, resp = "", mix = "", ...) {
+  stopifnot(bterms$family$link == "identity")
+  stopifnot(!isTRUE(nzchar(mix)))  # mixture models are not allowed
+  p <- stan_log_lik_dpars(bterms, TRUE, resp, mix, type = "multi")
+  p$Llncor <- glue("Llncor{mix}{resp}")
+  p$refcat <- get_refcat(bterms$family, int = TRUE)
+  sdist("logistic_normal_cholesky_cor", p$mu, p$sigma, p$Llncor, p$refcat)
+}
+
+stan_log_lik_ordinal <- function(bterms, resp = "", mix = "",
+                                 threads = NULL, ...) {
   prefix <- paste0(str_if(nzchar(mix), paste0("_mu", mix)), resp)
   p <- stan_log_lik_dpars(bterms, TRUE, resp, mix)
   if (use_ordered_logistic(bterms)) {
@@ -696,11 +783,12 @@ stan_log_lik_ordinal <- function(bterms, resp = "", mix = "", ...) {
     lpdf <- "ordered_logistic"
     p[grepl("^disc", names(p))] <- NULL
   } else {
-    lpdf <- paste0(bterms$family$family, "_", bterms$family$link) 
+    lpdf <- paste0(bterms$family$family, "_", bterms$family$link)
   }
   if (has_thres_groups(bterms)) {
     str_add(lpdf) <- "_merged"
-    p$Jthres <- paste0("Jthres", resp, "[n]")
+    n <- stan_nn(threads)
+    p$Jthres <- paste0("Jthres", resp, n)
     p$thres <- "merged_Intercept"
   } else {
     p$thres <- "Intercept"
@@ -711,7 +799,7 @@ stan_log_lik_ordinal <- function(bterms, resp = "", mix = "", ...) {
   }
   if (has_cs(bterms)) {
     if (has_thres_groups(bterms)) {
-      stop2("Cannot use category specific effects ", 
+      stop2("Cannot use category specific effects ",
             "in models with multiple thresholds.")
     }
     str_add(p$thres) <- paste0(" - transpose(mucs", prefix, "[n])")
@@ -737,7 +825,8 @@ stan_log_lik_hurdle_gamma <- function(bterms, resp = "", mix = "", ...) {
   p <- stan_log_lik_dpars(bterms, TRUE, resp, mix)
   usc_logit <- stan_log_lik_dpar_usc_logit("hu", bterms)
   lpdf <- paste0("hurdle_gamma", usc_logit)
-  sdist(lpdf, p$shape, p$mu, p$hu)
+  # Stan uses shape-rate parameterization for gamma with rate = shape / mean
+  sdist(lpdf, p$shape, paste0(p$shape, " / ", p$mu), p$hu)
 }
 
 stan_log_lik_hurdle_lognormal <- function(bterms, resp = "", mix = "", ...) {
@@ -747,7 +836,7 @@ stan_log_lik_hurdle_lognormal <- function(bterms, resp = "", mix = "", ...) {
   sdist(lpdf, p$mu, p$sigma, p$hu)
 }
 
-stan_log_lik_zero_inflated_poisson <- function(bterms, resp = "", mix = "", 
+stan_log_lik_zero_inflated_poisson <- function(bterms, resp = "", mix = "",
                                                ...) {
   p <- stan_log_lik_dpars(bterms, TRUE, resp, mix)
   lpdf <- stan_log_lik_simple_lpdf("zero_inflated_poisson", "log", bterms)
@@ -764,7 +853,7 @@ stan_log_lik_zero_inflated_negbinomial <- function(bterms, resp = "", mix = "",
 }
 
 stan_log_lik_zero_inflated_binomial <- function(bterms, resp = "", mix = "",
-                                                threads = 1, ...) {
+                                                threads = NULL, ...) {
   p <- stan_log_lik_dpars(bterms, TRUE, resp, mix)
   n <- stan_nn(threads)
   p$trials <- paste0("trials", resp, n)
@@ -774,6 +863,17 @@ stan_log_lik_zero_inflated_binomial <- function(bterms, resp = "", mix = "",
   sdist(lpdf, p$trials, p$mu, p$zi)
 }
 
+stan_log_lik_zero_inflated_beta_binomial <- function(bterms, resp = "",
+                                                     mix = "", threads = NULL,
+                                                     ...) {
+  p <- stan_log_lik_dpars(bterms, TRUE, resp, mix)
+  n <- stan_nn(threads)
+  p$trials <- paste0("trials", resp, n)
+  lpdf <- "zero_inflated_beta_binomial"
+  lpdf <- paste0(lpdf, stan_log_lik_dpar_usc_logit("zi", bterms))
+  sdist(lpdf, p$trials, p$mu, p$phi, p$zi)
+}
+
 stan_log_lik_zero_inflated_beta <- function(bterms, resp = "", mix = "", ...) {
   p <- stan_log_lik_dpars(bterms, TRUE, resp, mix)
   usc_logit <- stan_log_lik_dpar_usc_logit("zi", bterms)
@@ -781,7 +881,7 @@ stan_log_lik_zero_inflated_beta <- function(bterms, resp = "", mix = "", ...) {
   sdist(lpdf, p$mu, p$phi, p$zi)
 }
 
-stan_log_lik_zero_one_inflated_beta <- function(bterms, resp = "", mix = "", 
+stan_log_lik_zero_one_inflated_beta <- function(bterms, resp = "", mix = "",
                                                 ...) {
   p <- stan_log_lik_dpars(bterms, TRUE, resp, mix)
   sdist("zero_one_inflated_beta", p$mu, p$phi, p$zoi, p$coi)
@@ -795,10 +895,15 @@ stan_log_lik_zero_inflated_asym_laplace <- function(bterms, resp = "", mix = "",
   sdist(lpdf, p$mu, p$sigma, p$quantile, p$zi)
 }
 
-stan_log_lik_custom <- function(bterms, resp = "", mix = "", ...) {
-  # TODO: support reduce_sum
-  p <- stan_log_lik_dpars(bterms, TRUE, resp, mix)
+stan_log_lik_custom <- function(bterms, resp = "", mix = "", threads = NULL, ...) {
   family <- bterms$family
+  no_loop <- isFALSE(family$loop)
+  if (no_loop && (stan_log_lik_adj(bterms) || nzchar(mix))) {
+    stop2("This model requires evaluating the custom ",
+          "likelihood as a loop over observations.")
+  }
+  reqn <- !no_loop
+  p <- stan_log_lik_dpars(bterms, reqn, resp, mix)
   dpars <- paste0(family$dpars, mix)
   if (is_ordinal(family)) {
     prefix <- paste0(resp, if (nzchar(mix)) paste0("_mu", mix))
@@ -806,8 +911,14 @@ stan_log_lik_custom <- function(bterms, resp = "", mix = "", ...) {
   }
   # insert the response name into the 'vars' strings
   # addition terms contain the response in their variable name
+  n <- stan_nn(threads)
   var_names <- sub("\\[.+$", "", family$vars)
-  var_indices <- sub("^.+(?=\\[)", "", family$vars, perl = TRUE)
+  var_indices <- get_matches("\\[.+$", family$vars, first = TRUE)
+  has_n_index <- var_indices %in% "[n]"
+  if (no_loop && any(has_n_index)) {
+    stop2("Invalid use of index '[n]' in an unlooped custom likelihood.")
+  }
+  var_indices <- ifelse(has_n_index, n, var_indices)
   is_var_adterms <- var_names %in% c("se", "trials", "dec") |
     grepl("^((vint)|(vreal))[[:digit:]]+$", var_names)
   var_resps <- ifelse(is_var_adterms, resp, "")
@@ -822,17 +933,18 @@ use_glm_primitive <- function(bterms, allow_special_terms = TRUE) {
   stopifnot(is.brmsterms(bterms))
   # the model can only have a single predicted parameter
   # and no additional residual or autocorrelation structure
+  non_glm_adterms <- c("se", "weights", "thres", "cens", "trunc", "rate")
   mu <- bterms$dpars[["mu"]]
   if (!is.btl(mu) || length(bterms$dpars) > 1L ||
-      isTRUE(bterms$rescor) || length(bterms$adforms) ||
-      is.formula(mu$ac)) {
+      isTRUE(bterms$rescor) || is.formula(mu$ac) ||
+      any(names(bterms$adforms) %in% non_glm_adterms)) {
     return(FALSE)
   }
   # supported families and link functions
   # TODO: support categorical_logit primitive
   glm_links <- list(
     gaussian = "identity", bernoulli = "logit",
-    poisson = "log", negbinomial = "log"
+    poisson = "log", negbinomial = "log", negbinomial2 = "log"
     # rstan does not yet support 'ordered_logistic_glm'
     # cumulative = "logit"
   )
@@ -851,7 +963,7 @@ use_glm_primitive <- function(bterms, allow_special_terms = TRUE) {
 # @param bterms a btl object
 # @param resp optional name of the response variable
 # @return a named list of Stan code snippets
-args_glm_primitive <- function(bterms, resp = "", threads = 1) {
+args_glm_primitive <- function(bterms, resp = "", threads = NULL) {
   stopifnot(is.btl(bterms))
   decomp <- get_decomp(bterms$fe)
   center_X <- stan_center_X(bterms)
@@ -872,7 +984,7 @@ args_glm_primitive <- function(bterms, resp = "", threads = 1) {
       alpha <- glue("Intercept{resp}")
     } else {
       alpha <- "0"
-    } 
+    }
   }
   nlist(x, alpha, beta)
 }
@@ -881,7 +993,7 @@ args_glm_primitive <- function(bterms, resp = "", threads = 1) {
 use_ordered_logistic <- function(bterms) {
   stopifnot(is.brmsterms(bterms))
   isTRUE(bterms$family$family == "cumulative") &&
-    isTRUE(bterms$family$link == "logit") && 
+    isTRUE(bterms$family$link == "logit") &&
     isTRUE(bterms$fdpars$disc$value == 1) &&
     !has_cs(bterms)
 }
