@@ -10,6 +10,7 @@ test_that("posterior_epred helper functions run without errors", {
   fit <- add_dummy_draws(fit, "shape", dist = "exp")
   fit <- add_dummy_draws(fit, "alpha", dist = "norm")
   fit <- add_dummy_draws(fit, "hu", dist = "beta", shape1 = 1, shape2 = 1)
+  fit <- add_dummy_draws(fit, "phi", dist = "beta", shape1 = 1, shape2 = 1)
   fit <- add_dummy_draws(fit, "zi", dist = "beta", shape1 = 1, shape2 = 1)
   fit <- add_dummy_draws(fit, "quantile", dist = "beta", shape1 = 2, shape2 = 1)
   fit <- add_dummy_draws(fit, "xi", dist = "unif", min = -1, max = 0.5)
@@ -21,63 +22,75 @@ test_that("posterior_epred helper functions run without errors", {
   prep$dpars$nu <- brms:::get_dpar(prep, "nu")
   ndraws <- ndraws(fit)
   nobs <- nobs(fit)
-  
+
   # test preparation of truncated models
   prep$data$lb <- 0
   prep$data$ub <- 200
   mu <- brms:::posterior_epred_trunc(prep)
   expect_equal(dim(mu), c(ndraws, nobs))
-  
+
   # pseudo log-normal model
   fit$family <- fit$formula$family <- lognormal()
-  expect_equal(dim(posterior_epred(fit, summary = FALSE)), 
+  expect_equal(dim(posterior_epred(fit, summary = FALSE)),
                c(ndraws, nobs))
-  
+
   # pseudo shifted log-normal model
   fit$family <- fit$formula$family <- shifted_lognormal()
-  expect_equal(dim(posterior_epred(fit, summary = FALSE)), 
+  expect_equal(dim(posterior_epred(fit, summary = FALSE)),
                c(ndraws, nobs))
-  
+
   # pseudo skew-normal model
   fit$family <- fit$formula$family <- skew_normal()
-  expect_equal(dim(posterior_epred(fit, summary = FALSE)), 
+  expect_equal(dim(posterior_epred(fit, summary = FALSE)),
                c(ndraws, nobs))
-  
+
   # pseudo asym_laplace model
   fit$family <- fit$formula$family <- asym_laplace()
-  expect_equal(dim(posterior_epred(fit, summary = FALSE)), 
+  expect_equal(dim(posterior_epred(fit, summary = FALSE)),
                c(ndraws, nobs))
-  
+
   # pseudo zero_inflated_asym_laplace model
   fit$family <- fit$formula$family <- brmsfamily("zero_inflated_asym_laplace")
-  expect_equal(dim(posterior_epred(fit, summary = FALSE)), 
+  expect_equal(dim(posterior_epred(fit, summary = FALSE)),
                c(ndraws, nobs))
-  
+
   # pseudo gen_extreme_value model
   fit$family <- fit$formula$family <- gen_extreme_value()
-  expect_equal(dim(posterior_epred(fit, summary = FALSE)), 
+  expect_equal(dim(posterior_epred(fit, summary = FALSE)),
                c(ndraws, nobs))
-  
+
   # pseudo weibull model
   fit$formula$pforms <- NULL
   fit$family <- fit$formula$family <- weibull()
   expect_equal(dim(SW(posterior_epred(fit, summary = FALSE))), c(ndraws, nobs))
-  
+
   # pseudo binomial model
   fit$autocor <- brms:::cor_empty()
   fit$family <- fit$formula$family <- binomial()
   expect_equal(dim(SW(posterior_epred(fit, summary = FALSE))), c(ndraws, nobs))
   
+  # pseudo beta-binomial model
+  fit$family <- fit$formula$family <- beta_binomial()
+  expect_equal(dim(SW(posterior_epred(fit, summary = FALSE))), c(ndraws, nobs))
+
+  # pseudo zero inflated binomial model
+  fit$family <- fit$formula$family <- zero_inflated_binomial()
+  expect_equal(dim(SW(posterior_epred(fit, summary = FALSE))), c(ndraws, nobs))
+
+  # pseudo zero inflated beta binomial model
+  fit$family <- fit$formula$family <- zero_inflated_beta_binomial()
+  expect_equal(dim(SW(posterior_epred(fit, summary = FALSE))), c(ndraws, nobs))
+
   # pseudo hurdle poisson model
   fit$family <- fit$formula$family <- hurdle_poisson()
   fit$formula <- bf(count ~ Trt*Age + mo(Exp) + offset(Age) + (1+Trt|visit),
                     family = family(fit))
   expect_equal(dim(posterior_epred(fit, summary = FALSE)), c(ndraws, nobs))
-  
+
   # pseudo zero-inflated poisson model
   fit$family <- fit$formula$family <- zero_inflated_poisson()
   expect_equal(dim(posterior_epred(fit, summary = FALSE)), c(ndraws, nobs))
-  
+
   # pseudo custom model
   posterior_epred_test <- function(prep) {
     prep$dpars$mu
@@ -87,47 +100,47 @@ test_that("posterior_epred helper functions run without errors", {
     type = "int", vars = "trials[n]"
   )
   expect_equal(dim(posterior_epred(fit, summary = FALSE)), c(ndraws, nobs))
-  
+
   # truncated continuous models
   prep$dpars$shape <- c(as.matrix(fit, variable = "shape"))
   mu <- brms:::posterior_epred_trunc_gaussian(prep, lb = 0, ub = 10)
   expect_equal(dim(mu), c(ndraws, nobs))
-  
+
   mu <- brms:::posterior_epred_trunc_student(prep, lb = -Inf, ub = 15)
   expect_equal(dim(mu), c(ndraws, nobs))
-  
+
   mu <- brms:::posterior_epred_trunc_lognormal(prep, lb = 2, ub = 15)
   expect_equal(dim(mu), c(ndraws, nobs))
-  
+
   prep$dpars$mu <- exp(prep$dpars$mu)
   mu <- brms:::posterior_epred_trunc_gamma(prep, lb = 1, ub = 7)
   expect_equal(dim(mu), c(ndraws, nobs))
-  
+
   mu <- brms:::posterior_epred_trunc_exponential(prep, lb = 0, ub = Inf)
   expect_equal(dim(mu), c(ndraws, nobs))
-  
+
   mu <- SW(brms:::posterior_epred_trunc_weibull(prep, lb = -Inf, ub = Inf))
   expect_equal(dim(mu), c(ndraws, nobs))
-  
+
   # truncated discrete models
   data <- list(Y = sample(100, 10), trials = 1:10, N = 10)
   lb <- matrix(0, nrow = ndraws, ncol = nobs)
   ub <- matrix(100, nrow = ndraws, ncol = nobs)
   mu <- brms:::posterior_epred_trunc_poisson(prep, lb = lb, ub = ub)
   expect_equal(dim(mu), c(ndraws, nobs))
-  
+
   mu <- brms:::posterior_epred_trunc_negbinomial(prep, lb = lb, ub = ub)
   expect_equal(dim(mu), c(ndraws, nobs))
-  
+
   mu <- brms:::posterior_epred_trunc_negbinomial2(prep, lb = lb, ub = ub)
   expect_equal(dim(mu), c(ndraws, nobs))
-  
+
   mu <- brms:::posterior_epred_trunc_geometric(prep, lb = lb, ub = ub)
   expect_equal(dim(mu), c(ndraws, nobs))
-  
+
   prep$data$trials <- 120
   lb <- matrix(-Inf, nrow = ndraws, ncol = nobs)
-  prep$dpars$mu <- brms:::ilink(prep$dpars$mu, "logit")
+  prep$dpars$mu <- brms:::inv_link(prep$dpars$mu, "logit")
   mu <- brms:::posterior_epred_trunc_binomial(prep, lb = lb, ub = ub)
   expect_equal(dim(mu), c(ndraws, nobs))
 })
@@ -136,7 +149,7 @@ test_that("posterior_epred_lagsar runs without errors", {
   prep <- list(
     dpars = list(mu = matrix(rnorm(30), nrow = 3)),
     ac = list(
-      lagsar = matrix(c(0.3, 0.5, 0.7)), 
+      lagsar = matrix(c(0.3, 0.5, 0.7)),
       Msar = matrix(1:100, 10, 10)
     ),
     ndraws = 3,
@@ -160,7 +173,7 @@ test_that("posterior_epred for advanced count data distributions runs without er
   prep$family <- brmsfamily("discrete_weibull")
   pred <- suppressWarnings(brms:::posterior_epred_discrete_weibull(prep))
   expect_equal(dim(pred), c(ns, nobs))
-  
+
   prep$family <- brmsfamily("com_poisson")
   pred <- suppressWarnings(brms:::posterior_epred_com_poisson(prep))
   expect_equal(dim(pred), c(ns, nobs))
@@ -176,15 +189,16 @@ test_that("posterior_epred for multinomial and dirichlet models runs without err
     mu2 = array(rnorm(ns*nobs), dim = c(ns, nobs))
   )
   prep$data <- list(ncat = ncat, trials = sample(1:20, nobs))
- 
+  prep$refcat <- 1
+
   prep$family <- multinomial()
   pred <- brms:::posterior_epred_multinomial(prep = prep)
   expect_equal(dim(pred), c(ns, nobs, ncat))
-  
+
   prep$family <- dirichlet()
   pred <- brms:::posterior_epred_dirichlet(prep = prep)
   expect_equal(dim(pred), c(ns, nobs, ncat))
-  
+
   prep$family <- brmsfamily("dirichlet2")
   prep$dpars$mu1 <- array(rexp(ns*nobs, 1), dim = c(ns, nobs))
   prep$dpars$mu2 <- array(rexp(ns*nobs, 1), dim = c(ns, nobs))
@@ -196,7 +210,7 @@ test_that("posterior_epred for multinomial and dirichlet models runs without err
 test_that("posterior_epred() can be reproduced by using d<family>()", {
   fit4 <- rename_pars(brms:::brmsfit_example4)
   epred4 <- posterior_epred(fit4)
-  
+
   eta4 <- posterior_linpred(fit4)
   bprep4 <- prepare_predictions(fit4)
   thres4 <- bprep4$thres$thres
@@ -205,7 +219,7 @@ test_that("posterior_epred() can be reproduced by using d<family>()", {
   epred4_ch <- aperm(sapply(seq_len(dim(eta4)[2]), function(i) {
     dsratio(seq_len(ncol(thres4) + 1), eta4[, i, ], thres4, disc4[, i])
   }, simplify = "array"), perm = c(1, 3, 2))
-  
+
   expect_equivalent(epred4, epred4_ch)
 })
 
